import compiler, traceback, os.path

###########################################################################
## External API

def compile(filename):
    """
    Compile a Python source file to IMCC instructions
    """
    main=os.path.basename(filename).split('.')[0]

    push()
    output(".sub %s::__main__ @MAIN" % main)
    output("\t.param pmc sys::argv")
    execute(compiler.parseFile(filename))
    output(".end")
    pop()
   
###########################################################################
## Internal API

blocks=[]
def push():
    """
    Suspend current block, start a new block
    """
    blocks.append(block())

def pop():
    """
    Complete current block, resume previous block
    """
    print "\n".join(blocks.pop().instructions)
    if blocks: print

def lookup(symbol):
    """
    Lookup a symbol in the current scope
    """
    blocks[-1].lookup(symbol)

def execute(node):
    """
    execute (emit instructions for) a given compiler.ast node
    """
    map[node.__class__](node)

def evaluate(node):
    """
    evaluate (return symbol/constant/register for) a given compiler.ast node
    """
    return map[node.__class__](node)

def output(instruction):
    """
    output a single IMCC instruction
    """

    # pad right
    col=len(instruction)
    if instruction[0]=='\t': col+=7
    while col<24:
        instruction+='\t'
        col+=8

    # determine caller
    (file,line,func)=traceback.extract_stack()[-2][:3]
    file=os.path.basename(file).split('.')[0]

    # append to current block
    blocks[-1].append("%s\t# %s.%s:%d" % (instruction, file, func, line))

def fail(message):
    """
    report an error
    """
    print "***", message

###########################################################################

class block:
  def __init__(self):
    self.instructions=[]
    self.symbols=[]
  def lookup(self, symbol):
    if symbol not in self.symbols:
      output("\t.local pmc " + symbol)
      output("\t%s = new PerlInt" % symbol)
      self.symbols.append(symbol)
  def append(self, instruction):
    self.instructions.append(instruction)

#
# create a map between compiler.ast classes and pyrate.ast process functions
# for efficient dispatch
#
map={}
for node in compiler.ast.klasses.keys():
    if node[0].isupper() and node[1].islower():
        try:
            process=getattr(getattr(__import__("ast."+node),node),"process")
            map[compiler.ast.klasses[node]]=process
        except:
            print node
